from django.db import models
from django.contrib.auth.models import User
from django.conf import settings
from PIL import Image, ImageOps, ImageDraw
import os
		
class InstrumentType(models.Model):
	name = models.CharField(max_length=30)
	
	def __str__(self):
		return self.name
		
class InstrumentBrand(models.Model):
	name = models.CharField(max_length=30)
	
	def __str__(self):
		return self.name
		
class InstrumentModel(models.Model):
	name = models.CharField(max_length=30)
	type = models.ForeignKey(InstrumentType)
	brand = models.ForeignKey(InstrumentBrand)
	
	def __str__(self):
		return self.name
		
class UserInstrument(models.Model):
	user = models.ForeignKey(User)
	instrument = models.ManyToManyField(InstrumentModel)
	
	def __str__(self):
		return self.user.username

class Band(models.Model):
	name = models.CharField(max_length=30)
	members = models.ManyToManyField(User)
	
	def __str__(self):
		return self.name

class UserBand(models.Model):
	user = models.ForeignKey(User)
	bands = models.ManyToManyField(Band)
	
	def __str__(self):
		return self.user.username
		
class BandRequest(models.Model):
	user = models.ForeignKey(User)
	band = models.ForeignKey(Band)
	request_username = models.CharField(max_length=30)
	
	def __str__(self):
		return self.user.username
		
class PracticeLocal(models.Model):
	name = models.CharField(max_length=30)
	
	def __str__(self):
		return self.name

class BandTimetable(models.Model):
	band = models.ForeignKey(Band)
	local = models.ForeignKey(PracticeLocal)
	dia = models.SmallIntegerField()
	hora_inicio = models.SmallIntegerField()
	hora_fin = models.SmallIntegerField()
	
	def __str__(self):
		return self.band.name+"-"+self.local.name

def recortar(fname,trozo_izquierda,trozo_derecha,trozo_arriba,trozo_abajo,fname_recorte):
	try:
		img = Image.open(fname)
	except:
		print 'Unable to open ' + fname
	
	#Recorte
	width=img.size[0]
	height=img.size[1]
	recorte_w=width-float(trozo_izquierda)-float(trozo_derecha)
	recorte_h=height-float(trozo_arriba)-float(trozo_abajo)
	
	marco = [float(trozo_izquierda),float(trozo_arriba),width-float(trozo_derecha),height-float(trozo_abajo)]
	recorte = img.crop(marco)
    
	try:
		recorte.save(fname_recorte)
		size = []
		size.append(recorte_w)
		size.append(recorte_h)
		return size
	except:
		print 'Unable to save in ' + fname_recorte

def scaleWithFactor(fname, scalingFactor, fname_scaled):
	try:
		img = Image.open(fname)
	except:
		print 'Unable to open ' + fname

	# We check if size is suitable for saving
	width  = int(img.size[0] * float(scalingFactor))
	height = int(img.size[1] * float(scalingFactor))
	
	img = img.resize((width, height), Image.ANTIALIAS)
    
	try:
		img.save(fname_scaled)
	except:
		print 'Unable to save in ' + fname_scaled

def scale(fname, maxWidth, maxHeight, fname_scaled):
	try:
		img = Image.open(fname)
	except:
		print 'Unable to open ' + fname

	# We check if size is suitable for saving
	width  = img.size[0]
	height = img.size[1]
	
	scalingFactor = 1.0

	# max/min cannot have more difference than 16:9
	if (float(max(width, height))/float(min(width,height)) < 16.0/9.0):
		if max(width, height) > max(maxWidth, maxHeight):
			m = max(maxWidth, maxHeight)+0.0
			scalingFactor = m/max(width, height)
		width  = int(width  * scalingFactor)
		height = int(height * scalingFactor)
	else: # If proportions are not as desired, we just deform image
		if width > maxWidth:
			width = maxWidth
		if height > maxHeight:
			height = maxHeight
	
	img = img.resize((width, height))
    
	try:
		img.save(fname_scaled)
		return img
	except:
		print 'Unable to save in ' + fname_scaled
		
class Resource(models.Model):
	onCreate = models.DateTimeField(auto_now = False,auto_now_add=True)
	type = models.CharField( max_length=5 )
	file = models.FileField( null=False, upload_to="resources/imagenes/faces" )

	def save(self):
		super(Resource,self).save()
        
		#Encontramos la extension buscando el primer punto por la derecha
		self.type = 'jpg' #self.file.name[ self.file.name.rfind(".")+1:len(self.file.name)].lower();
		
		#Para cada tipo de archivo, buscamos si existe en alguno de nuestros tipos definidos
		#Si es asi, creamos un subdirectorio en la carpeta sources
		#for t in settings.RESOURCES_TYPES:
			#if self.type in settings.RESOURCES_TYPES[t]:
				#subDir = t
				
		#newName = os.path.join(os.path.dirname(self.file.path), subDir)
		#newName = os.path.dirname(self.file.path)
		
		#if not os.path.exists(newName):
			#os.mkdir(newName)
		newName = os.path.join(os.path.dirname(self.file.path), str(self.id) +'.'+self.type )
        
		#if self.type in settings.RESOURCES_TYPES['img']:
		img = scale(self.file.path,200,200,newName)
		img.save(os.path.join(os.path.dirname(self.file.path), str(self.id) +'_tmp.'+self.type ))
		os.remove(self.file.path)
		#else:
		#os.rename(self.file.path, newName)
      
		#self.file.name = os.path.join( os.path.dirname(self.file.name), subDir , os.path.basename(newName))
		self.file.name = os.path.join( os.path.dirname(self.file.name), os.path.basename(newName))

		super(Resource,self).save()
		
		return self.id
	
	def __str__(self):
		return self.id
		
class UserTempPhoto(models.Model):
	user = models.ForeignKey(User)
	resource = models.ForeignKey(Resource)

	def __str__(self):
		return self.user.username

class Appearance(models.Model):
	resource = models.ForeignKey(Resource)
	face_x = models.FloatField()
	face_y = models.FloatField()
	face_w = models.FloatField()
	face_h = models.FloatField()
	escala = models.FloatField()
	user = models.ForeignKey(User)
	
	def __str__(self):
		return self.user.username
		
class FamousArtist(models.Model):
	name = models.CharField( max_length=30 )
	
	def __str__(self):
		return self.name
		
class Song(models.Model):
	tittle = models.CharField( max_length=30 )
	
	def __str__(self):
		return self.tittle
		
class Album(models.Model):
	tittle = models.CharField( max_length=30 )
	artist = models.ForeignKey(FamousArtist)
	songs = models.ManyToManyField(Song)

	def __str__(self):
		return self.tittle

class Concert(models.Model):
	band = models.ForeignKey(Band)
	fecha = models.DateTimeField()
	play_list = models.ManyToManyField(Song)
	public = models.ManyToManyField(User)

	def __str__(self):
		return self.band.name+" "+str(self.fecha)
		
class ConcertRequest(models.Model):
	user = models.ForeignKey(User)
	concert = models.ForeignKey(Concert)
	request_username = models.CharField(max_length=30)
	
	def __str__(self):
		return self.user.username+" "+self.concert.band.name+" "+str(self.concert.fecha)